//////////////////////////////////////////////////////////////////////
//+------------------------------------------------------------------+
//|                                                      DCT_EYE.mq4 |
//|                                      Copyright  2005, Alexander |
//|                                        mailto:a_g_j_@hotmail.com |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
//---- general part
#property copyright "Copyright  2005, Alexander"
#property link      "mailto:a_g_j_@hotmail.com"
//---- indicator properties
#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color1 Aqua
//---- defines
#define CountBarsMax 1000 //     
#define Pi 3.1415926535 //   :)
#define epsilon 0.000000001 // 1e-09
//---- input parameters
extern int       CountBars = 360; //    dct-
extern int       CountTicks = 0; //   ; 0 - 
extern double    EyeSize = 1.5; //  ""
extern double    Alpha = 0.2; //     
extern int PriceApplied = 5; // ,    :
                             // 1 - Close, 2 - Open, 3 - High, 4 - Low,
                             // 5 - Median ( , (High+Low)/2,
                             // 6 - Typical (, (High+Low+Close)/3,
                             // 7 - Weighted (  ,
                             //     (High+Low+Close+Close)/4,
                             // 8 - General Mean, (High+Open+Low+Close)/4
extern bool Smooth_X = true; //  .  ("./")
extern bool Sound_On = false; //     ("./")
//---- buffers
double ExtMapBuffer1[]; // 1-  
double CoeffDCT[][CountBarsMax]; //   
double x[]; //   dct- -> y[]
double y[]; //   idct- -> x[]
double kern[]; //   Eye
double aux_x[]; //  
double aux_y[]; //  
//---- variables
int N; // N = CountBars
int nticks; //  
double price_max; //    CountBars 
double price_min; //    CountBars 
double eyesizeA; // abs(EyeSize)
double alphaA; // abs(Alpha)
double delta; // 2*Pi/(CountBars-1)
double wgt_zero; //       k = 0
double wgt_nzero; //       1 <= k <= N-1
bool IniCheckFailed; //     
bool RunOnce; //   
//////////////////////////////////////////////////////////////////////
//+------------------------------------------------------------------+
//|                                                 |
//| (    init())                     |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
void OnInitializationChecks()
 {
 //----
//
   if ((PriceApplied < 1) || (PriceApplied > 8)) //   
    {
     Alert("  . : 1 - Close, ",
           "\n", "2 - Open, 3 - High, 4 - Low, 5 - Median, 6 - Typical, ",
           "\n", "7 - WeightedClose, 8 - General Mean.");
     return;
    }
// 
  if (Bars == 0) //        
   {
    Print("---- Bar Chart has been renewed. ----");
    return;
   }
//
  if (Bars < CountBars) //      
   {
    Alert("       .",
          "\n", "      ", Bars, ".");
    IniCheckFailed = true; //    
    return;
   }
//    
  if (CountBars < 10) //     .
   {
    Alert("      .",
          "\n", "     10 .");
    IniCheckFailed = true; //    
    return;
   }
//    
  if (CountBars > CountBarsMax) //     .
   {
    Alert("     .",
          "\n", "     ", CountBarsMax, ".");
    IniCheckFailed = true; //    
    return;
   }
//      
  if (CountTicks < 0) //    
   {
    Alert("   .", "\n",
          "    0.");
    IniCheckFailed = true; //    
    return;    
   }
//   
//   Print("---- On initialization checks successful. ----"); 
 //----
  return;
 }
//////////////////////////////////////////////////////////////////////
//+------------------------------------------------------------------+
//|                                      |
//| (    init())                     |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
void InitOpArrays()
 {
 //----
  int ArrSize; //
//     ,  
  if (IniCheckFailed == true) return;     
//  
//   x[]
  ArrSize = ArrayResize(x, N);
  ArrSize = ArrayInitialize(x, 0);
//   y[]
  ArrSize = ArrayResize(y, N);
  ArrSize = ArrayInitialize(y, 0);
//   kern[]
  ArrSize = ArrayResize(kern, N);
  ArrSize = ArrayInitialize(kern, 0);
//   aux_x[]
  ArrSize = ArrayResize(aux_x, N);
  ArrSize = ArrayInitialize(aux_x, 0);
//   aux_y[]
  ArrSize = ArrayResize(aux_y, N);
  ArrSize = ArrayInitialize(aux_y, 0);  
//
//  Print("---- Operational arrays initialized. ----");  
 //----
  return;
 }
//////////////////////////////////////////////////////////////////////
//+------------------------------------------------------------------+
//|    -          |
//| (    init())                     |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
void GetCoeffsDCT()
 {
 //----
//   
   int k; //    
   int n; //    
   double PiN2; // Pi/(N*2)
   int ArrSize; //
//   
//     ,  
   if (IniCheckFailed == true) return;   
//    
   PiN2 = Pi / (N * 2.);
   wgt_zero = 1. / MathSqrt(N); //   0
   wgt_nzero = MathSqrt(2.) * wgt_zero; //    
//   CoeffDCT[][]
   ArrSize = ArrayResize(CoeffDCT, N);
   ArrSize = ArrayInitialize(CoeffDCT, 0);
//   
   for (k=0; k<=N-1; k++)
    { //   k
     for (n=0; n<=N-1; n++)
      { //   n
       CoeffDCT[k][n] = MathCos(PiN2 * (2 * n + 1) * k);
      } //   n
    } //   k
//
//   Print("---- Coefficient matrix elements calculated. ----");    
 //----
   return;
  }
//////////////////////////////////////////////////////////////////////
//+------------------------------------------------------------------+
//|    ( )                    |
//| (    init())                     |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
void GetKern()
 {
 //----
// 
  int i; // 
  double domega;
//  
//     ,  
  if (IniCheckFailed == true) return;   
//
  for (i=0; i<N; i++)  
   {
    domega = delta * i * eyesizeA;
    kern[i] = (MathSin(domega) + epsilon) / (domega + epsilon);
   }
//   
 //----
  return;
 }
//////////////////////////////////////////////////////////////////////
//+------------------------------------------------------------------+
//|   - (DCT) x[] -> y[]        |
//| (      )           |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
void DCT()
 {
 //----
   int k; //    
   int n; //    
   double sum; // 
// dct-
   for (k=0; k<=N-1; k++)
    { //   k
     sum = 0; //      
     for (n=0; n<=N-1; n++)
      { //   n
       sum = sum + x[n] * CoeffDCT[k][n];
      } //   n
       if (k != 0)
       {
        y[k] = wgt_nzero * sum;
       }
       else
       {
        y[k] = wgt_zero * sum;
       }            
    } //   k
 //----
  return;
 }
////////////////////////////////////////////////////////////////////// 
//+------------------------------------------------------------------+
//|   - (IDCT) y[] -> x[]     |
//| (      )           |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
void IDCT()
 {
 //----
   int k; //    
   int n; //    
   double sum; // 
// idct-
   for (n=0; n<=N-1; n++)
    { //   n
     sum = 0; //      
     for (k=0; k<=N-1; k++)
      { //   k
       if (k != 0)
        sum = sum + wgt_nzero * y[k] * CoeffDCT[k][n];      
       else
        sum = sum + wgt_zero * y[k] * CoeffDCT[k][n];      
       } //   k
     x[n] = sum;
    } //   n
 //----
  return;
 }
//////////////////////////////////////////////////////////////////////   
//+------------------------------------------------------------------+
//|   y[] -             |
//| (      )           |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
void ArraysProcessing()
 {
 //----
// 
  int i; // 
  double sqomega; // 1 + omega^2
//  
//     -    :  
// y[.]=kern[.].*y[.]/(kern[.].^2+alpha*(1+omega^2)) -> x[.]=idct(y[.])
  for (i=0; i<N; i++)
   {
    sqomega = 1 + (i * delta) * (i * delta); // 1 + omega^2
    y[i] = (kern[i] * y[i]) / (kern[i] * kern[i] + alphaA * sqomega);
   } 
//      
 //----
  return;
 }
 //////////////////////////////////////////////////////////////////////   
//+------------------------------------------------------------------+
//|           |
//| (      )           |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
void NormArray(double p_max, double p_min)
 //----
 {
  double x_max; //
  double x_min; //
  double x_curr; //
  double coef_1; //
  double coef_2; //
  int i; // 
//       
  x_min = 9999.;
  x_max = 0;
  for (i=0; i<N; i++)
   {
    x_curr = aux_x[i];
    if (x_curr >= x_max) x_max = x_curr;
    if (x_curr <= x_min) x_min = x_curr;
   }
//      
  coef_1 = (p_min * x_max - p_max * x_min) / (x_max - x_min);
  coef_2 = (p_max - p_min) / (x_max - x_min);
//     
   for (i=0; i<N; i++)
    {
     x_curr = aux_x[i];
     aux_x[i] = coef_1 + x_curr * coef_2;
    }   
 //----
  return;
 }
//////////////////////////////////////////////////////////////////////
//+------------------------------------------------------------------+
//| C   5                                   |
//| (      )           |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
void SmoothArray()
 {
 //----
  int i; // 
//  
//    5 
  aux_y[0] = (3 * aux_x[0] + 2 * aux_x[1] + aux_x[2] - aux_x[4]) / 5.;
  aux_y[1] = (4 * aux_x[0] + 3 * aux_x[1] + 2 * aux_x[2] + aux_x[3]) / 10.;
  for (i=2; i<=N-3; i++) 
   {
    aux_y[i] = (aux_x[i-2] + aux_x[i-1] + aux_x[i] + 
                aux_x[i+1] + aux_x[i+2]) / 5.;
   }
  aux_y[N-2] = (aux_x[N-4] + 2 * aux_x[N-3] + 
                3 * aux_x[N-2] + 4 * aux_x[N-1]) / 10.;
  aux_y[N-1] = (3 * aux_x[N-1] + 2 * aux_x[N-2] + 
                aux_x[N-3] - aux_x[N-4]) / 5.;
//
  ArrayCopy(aux_x, aux_y, 0, 0, N-1); //  N-   
                                      // (" ")
 //----
  return;
 } 
//////////////////////////////////////////////////////////////////////
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
int init()
  {
//  
   if (RunOnce == true) return(0); //   
   IniCheckFailed = false; //    
   RunOnce = false; //    
//   
   N = CountBars; //   
   nticks = 0; //   
   alphaA = MathAbs(Alpha); //   
   eyesizeA = MathAbs(EyeSize); // ----//----
   delta = 2 * Pi / (N - 1); //  
//         
   OnInitializationChecks();
//     :
   RunOnce = true; //    ;
   GetCoeffsDCT(); //    dct-;
   InitOpArrays(); //   ;
   GetKern();      //  .
//---- indicators drawing settings
   IndicatorDigits(MarketInfo(Symbol(), MODE_DIGITS));
   IndicatorShortName("DCT_Eye_IE");   
//   
   SetIndexStyle(0,DRAW_LINE,EMPTY,1);
   SetIndexLabel(0,"DCT_Eye");   
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexDrawBegin(0,0);
//----
   return(0);
  }
//////////////////////////////////////////////////////////////////////  
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
int deinit()
  {
//----
 
   switch(UninitializeReason())
    {
     case REASON_CHARTCLOSE:  
                              break;
//     
     case REASON_REMOVE:      //CleanUp(); 
                              break; //    
     case REASON_RECOMPILE:   
                              break;
//     
     case REASON_CHARTCHANGE: 
                              break;
//     
     case REASON_PARAMETERS:  
                              break;
//     
     case REASON_ACCOUNT:     //StoreData(); 
                              break; //   
    }
    
//----
   return(0);
  }
//////////////////////////////////////////////////////////////////////  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////
int start()
  {
   int counted_bars = IndicatorCounted();
   int i; // 
   double price_curr;
//---- 
//     ,  
   if (IniCheckFailed == true) return;
//
   nticks++; //   
   if (nticks < CountTicks) return(0); //  ,  
   nticks = 0; //   
//    ,     
   if (Sound_On == true) PlaySound("tick.wav");   
//      
//        
   price_min = 9999;
   price_max = 0;   
   switch (PriceApplied)
    {
     case 1: for (i=N-1; i>=0; i--)
              { // Close Price
               price_curr = Close[i];
               x[i] = price_curr;
               if (price_curr >= price_max) price_max = price_curr;
               if (price_curr <= price_min) price_min = price_curr;
              }
             break;
     case 2: for (i=N-1; i>=0; i--)
              { // Open Price
               price_curr = Open[i];
               x[i] = price_curr;
               if (price_curr >= price_max) price_max = price_curr;
               if (price_curr <= price_min) price_min = price_curr;               
              }
             break;
     case 3: for (i=N-1; i>=0; i--)
              {// High Price
               price_curr = High[i];
               x[i] = price_curr;
               if (price_curr >= price_max) price_max = price_curr;
               if (price_curr <= price_min) price_min = price_curr;               
              }
             break;
     case 4: for (i=N-1; i>=0; i--)
              {// Low Price
               price_curr = Low[i];
               x[i] = price_curr;
               if (price_curr >= price_max) price_max = price_curr;
               if (price_curr <= price_min) price_min = price_curr;               
              }
             break;
     case 5: for (i=N-1; i>=0; i--)
              { // Median Price, (High+Low)/2
               price_curr = (High[i] + Low[i]) / 2;
               x[i] = price_curr;
               if (price_curr >= price_max) price_max = price_curr;
               if (price_curr <= price_min) price_min = price_curr;               
              }
             break;
     case 6: for (i=N-1; i>=0; i--)
              { // Typical Price, (High+Low+Close)/3
               price_curr = (High[i] + Low[i] + Close[i]) / 3;
               x[i] = price_curr;
               if (price_curr >= price_max) price_max = price_curr;
               if (price_curr <= price_min) price_min = price_curr;               
              }
             break;
     case 7: for (i=N-1; i>=0; i--)
              { // Weighted Close Price, (High+Low+Close+Close)/4
               price_curr = (High[i] + Low[i] + 2 * Close[i]) / 4;
               x[i] = price_curr;
               if (price_curr >= price_max) price_max = price_curr;
               if (price_curr <= price_min) price_min = price_curr;
              }                                   
             break;
     case 8: for (i=N-1; i>=0; i--)
              { // General Mean Price, (High+Open+Low+Close)/4
               price_curr = (High[i] + Open[i] + Low[i] + Close[i]) / 4;
               x[i] = price_curr;
               if (price_curr >= price_max) price_max = price_curr;
               if (price_curr <= price_min) price_min = price_curr;               
              }                                   
             break;             
    }            
//    
   DCT(); // dct-   x[] -> y[i]
//
   ArraysProcessing(); //     y[]
//    
   IDCT(); // idct-    [] -> x[]
//   
   ArrayCopy(aux_x, x); //  x[]   
   NormArray(price_max, price_min); //  x[]
   if (Smooth_X == true) //   x[] 
    SmoothArray(); //  x[]       
   ArrayCopy(x, aux_x); //  x[]    
//
   ArrayCopy(ExtMapBuffer1, x); //  x[]  1-  
//
//----
   return(0);
  }
//+------------------------------------------------------------------+
//////////////////////////////////////////////////////////////////////

